"""Helper functions

This module is available as the ``h`` variable in templates, and can also be
imported into handler modules or elsewhere. Put any common functions you want
to access in all templates here.

The WebHelpers package (http://python.org/pypi/WebHelpers) contains some
commonly used helpers for HTML tag creation, text formatting, number
formatting, etc.

The template globals (``h`` et al) are set in
``{{package}}.subscribers.add_renderer_globals()``.
"""

#from webhelpers.html import *
#from webhelpers.html.tags import *
